@@ -27,6 +27,7 @@
 #include <sys/stat.h>
 #include <vector>
 #include <map>
+#include <set>
 #include <utility> //for pair
 #include <boost/dynamic_bitset.hpp>
 
@@ -45,10 +46,12 @@ template<typename Integer> class Full_Cone;
 //template<typename Integer> class Matrix;
 
 // type for simplex, short in contrast to class Simplex
-template<typename Integer> struct SHORTSIMPLEX {
+template<typename Integer> 
+struct SHORTSIMPLEX {
     vector<key_t> key;                // full key of simplex
     Integer height;                   // height of last vertex over opposite facet
     Integer vol;                      // volume if computed, 0 else
+    Integer vol_for_detsum;           // used by QuFullCone
     vector<bool> Excluded;           // for disjoint decomposition of cone
                                       // true in position i indictate sthat the facet 
                                       // opposite of generator i must be excluded
@@ -292,6 +295,15 @@ public:
     const Matrix<Integer>& getDeg1ElementsMatrix();
     const vector< vector<Integer> >& getDeg1Elements();
     size_t getNrDeg1Elements();
+    
+    size_t getNumberLatticePoints();
+    
+    const Matrix<Integer>& getLatticePointsMatrix();
+    const vector< vector<Integer> >& getLatticePoints();
+    size_t getNrLatticePoints();
+    
+    const map<boost::dynamic_bitset<>,int>& getFaceLattice();
+    vector<size_t> getFVector();
 
     // the actual grading is Grading/GradingDenom
     vector<Integer> getGrading();
@@ -303,9 +315,12 @@ public:
 
     mpq_class getMultiplicity();
     mpq_class getVolume();
+    renf_elem_class getRenfVolume();
     nmz_float getEuclideanVolume();
     mpq_class getVirtualMultiplicity();
     mpq_class getIntegral();
+    nmz_float getEuclideanIntegral();
+
     const pair<HilbertSeries, mpz_class>& getWeightedEhrhartSeries();
     
     string getPolynomial() const;
@@ -326,6 +341,7 @@ public:
 
     const Sublattice_Representation<Integer>& getSublattice();
     const HilbertSeries& getHilbertSeries(); //general purpose object
+    const HilbertSeries& getEhrhartSeries(); //general purpose object
 
     // the following 2 methods give information about the last used triangulation
     // if no triangulation was computed so far they return false
@@ -345,6 +361,9 @@ public:
     void setPolynomial(string poly);
     void setNrCoeffQuasiPol(long nr_coeff);
     void setExpansionDegree(long degree);
+    void setFaceCodimBound(long bound);
+    
+    void setRenf(renf_class *renf);
     
     bool get_verbose ();
     
@@ -367,10 +386,14 @@ public:
     mpq_class getRationalConeProperty(ConeProperty::Enum property);
 
     nmz_float getFloatConeProperty(ConeProperty::Enum property);
+    
+    renf_elem_class getFieldElemConeProperty(ConeProperty::Enum property);
 
     size_t getMachineIntegerConeProperty(ConeProperty::Enum property);
 
     bool getBooleanConeProperty(ConeProperty::Enum property);
+    
+    nmz_float euclidean_corr_factor();
 
 //---------------------------------------------------------------------------
 //                          private part
@@ -400,6 +423,7 @@ private:
     // Matrix<Integer> GeneratorsOfToricRing;
     Matrix<Integer> OriginalMonoidGenerators;
     Matrix<Integer> Generators;
+    Matrix<Integer> ReferenceGenerators;
     Matrix<Integer> ExtremeRays;
     Matrix<nmz_float> VerticesFloat;
     vector<bool> ExtremeRaysIndicator;
@@ -420,7 +444,9 @@ private:
     list< STANLEYDATA<Integer> > StanleyDec_export;
     mpq_class multiplicity;
     mpq_class volume;
-    double euclidean_volume;
+    nmz_float euclidean_volume;
+    nmz_float euclidean_height; // for volume computations wuth renf_elem_class
+    renf_elem_class renf_volume;
     mpq_class Integral;
     mpq_class VirtualMultiplicity;
     vector<Integer> WitnessNotIntegrallyClosed;
@@ -431,20 +457,27 @@ private:
     Matrix<Integer> ModuleGeneratorsOverOriginalMonoid;
     Matrix<Integer> Deg1Elements;
     HilbertSeries HSeries;
+    HilbertSeries EhrSeries;
     IntegrationData IntData;
     vector<Integer> Grading;
     vector<Integer> Dehomogenization;
+    vector<Integer> Norm; // used by v_standardize in the numberfield case
     Integer GradingDenom;
     Integer index;  // the internal index
     Integer unit_group_index;
+    size_t number_lattice_points;
+    vector<size_t> f_vector;
     
     vector<boost::dynamic_bitset<> > Pair; // for indicator vectors in project-and_lift
     vector<boost::dynamic_bitset<> > ParaInPair; // if polytope is a parallelotope
     bool check_parallelotope();
     bool is_parallelotope;
+    
+    map<boost::dynamic_bitset<>,int> FaceLattice;
 
     bool pointed;
     bool inhomogeneous;
+    bool polytope_in_input;
     bool gorensetin;
     bool deg1_extreme_rays;
     bool deg1_hilbert_basis;
@@ -467,6 +500,14 @@ private:
 
     bool no_lattice_restriction; // true if cine generators are known to be in the relevant lattice
     bool normalization; // true if input type normalization is used
+    bool general_no_grading_denom;
+    
+#ifdef ENFNORMALIZ    
+    renf_class *Renf;
+#endif
+    
+    long renf_degree;
+    long face_codim_bound;
 
     // if this is true we allow to change to a smaller integer type in the computation
     bool change_integer_type;
@@ -495,6 +536,8 @@ private:
     void check_precomputed_support_hyperplanes();
     void check_excluded_faces();
     
+    void check_gens_vs_reference(); //to make sure that newly computed generators agrre with the previously computed
+    
     void setGrading (const vector<Integer>& lf);
     void setWeights ();
     void setDehomogenization (const vector<Integer>& lf);
@@ -505,6 +548,11 @@ private:
     
     void try_symmetrization(ConeProperties& ToCompute);
     void try_approximation_or_projection (ConeProperties& ToCompute);
+    
+    void try_Hilbert_Series_from_lattice_points(const ConeProperties& ToCompute);
+    void make_Hilbert_series_from_pos_and_neg(const vector<num_t>& h_vec_pos, const vector<num_t>& h_vec_neg);
+    
+    void make_face_lattice(const ConeProperties& ToCompute);
 
     Matrix<Integer> prepare_input_type_2(const vector< vector<Integer> >& Input);
     Matrix<Integer> prepare_input_type_3(const vector< vector<Integer> >& Input);
@@ -530,7 +578,7 @@ private:
 
     /* extract the data from Full_Cone, this may remove data from Full_Cone!*/
     template<typename IntegerFC>
-    void extract_data(Full_Cone<IntegerFC>& FC);
+    void extract_data(Full_Cone<IntegerFC>& FC,ConeProperties& ToCompute);
     template<typename IntegerFC>
     void extract_supphyps(Full_Cone<IntegerFC>& FC, Matrix<Integer>& ret, bool dual=true);
     void extract_supphyps(Full_Cone<Integer>& FC, Matrix<Integer>& ret, bool dual=true);    
@@ -578,12 +626,13 @@ private:
     template<typename IntegerFC>
     void give_data_of_approximated_cone_to(Full_Cone<IntegerFC>& FC);
     
-    void project_and_lift(ConeProperties& ToCompute, Matrix<Integer>& Deg1, const Matrix<Integer>& Gens, Matrix<Integer>& Supps, bool float_projection);
+    void project_and_lift(const ConeProperties& ToCompute, Matrix<Integer>& Deg1, const Matrix<Integer>& Gens, 
+                          const Matrix<Integer>& Supps, const Matrix<Integer>& Congs, const vector<Integer> GradingOnPolytope); 
 
     void compute_volume(ConeProperties& ToCompute);
-    void compute_euclidean_volume(const vector<Integer>& Grad, Integer GradDenom);
     
     void try_multiplicity_by_descent(ConeProperties& ToCompute);
+    void try_multiplicity_of_para(ConeProperties& ToCompute);
     
     void compute_projection(ConeProperties& ToCompute);
     void compute_projection_from_gens(const vector<Integer>& GradOrDehom);
@@ -592,6 +641,9 @@ private:
     //in order to avoid getRank fromm inside compute
     size_t get_rank_internal();
     const Sublattice_Representation<Integer>& get_sublattice_internal();
+    
+    void compute_lattice_points_in_polytope(ConeProperties& ToCompute);
+    void prepare_volume_computation(ConeProperties& ToCompute);
 };
 
 // helpers
@@ -609,7 +661,7 @@ void insert_column(vector< vector<Integer> >& mat, size_t col, Integer entry);
 // computes approximating lattice simplex using the A_n dissection of the unit cube
 // q is a rational vector with the denominator in the FIRST component q[0]
 template<typename Integer>
-void approx_simplex(const vector<Integer>& q, std::list<vector<Integer> >& approx, const long approx_level){
+inline void approx_simplex(const vector<Integer>& q, std::list<vector<Integer> >& approx, const long approx_level){
 	
 	//cout << "approximate the point " << q;
     long dim=q.size();
@@ -677,6 +729,10 @@ void approx_simplex(const vector<Integer>& q, std::list<vector<Integer> >& appro
 
 }
 
+template<>
+inline void approx_simplex(const vector<renf_elem_class>& q, std::list<vector<renf_elem_class> >& approx, const long approx_level){
+    assert(false);    
+}
 
 }  //end namespace libnormaliz
 
